#ifndef _BUF_H
#define _BUF_H

/* line buffer */

typedef struct buf_line buf_line_t;

/* line buffer callback */
typedef void (* buf_line_call_t) (void *data, char *line, int len);

/* create new line buffer */
buf_line_t *buf_line_new(void *data, buf_line_call_t cb);

/* change data and callback */
void buf_line_setcb(buf_line_t *lb, void *data, buf_line_call_t cb);

/* free line buffer */
void buf_line_free(buf_line_t *lb);

/* write data into line buffer
 * callback gets called with new lines
 * returns number of new lines
 * EOLs are cut off
 * final \0 (in place of EOL) is guaranteed */
int buf_line_write(buf_line_t *lb, const char *buf, int len);

/* packet fifo */

typedef struct buf_fifo buf_fifo_t;

/* create new fifo
 * packet gives packet size */
buf_fifo_t *buf_fifo_new(int packet);

/* free fifo */
void buf_fifo_free(buf_fifo_t *bf);

/* write into fifo */
void buf_fifo_write(buf_fifo_t *bf, const char *buf, int len);

/* get next available packet size
 * if 0, fifo is empty */
int buf_fifo_packetsize(buf_fifo_t *bf);

/* get next available packet
 * writes pointer to packet to buf
 * returns packet size
 * packet is not removed from fifo */
int buf_fifo_getpacket(buf_fifo_t *bf, char **buf);

/* remove bytes from fifo
 * (cannot be more than the available packet) */
void buf_fifo_advance(buf_fifo_t *bf, int n);

#endif
