typedef struct {
	int h;
	int s;
} hspair_t;

/* thread for wrapping a HANDLE in a SOCKET:
 * reading from handle, writing into socket */
static DWORD os_wrap_read_run(LPVOID data)
{
	hspair_t *p = data;
	int h = p->h;
	int s = p->s;
	int err;

	free(p);

	char buf[2048];
	DWORD r, at, w;

	//fprintf(stderr, "entering os_wrap_read %d %d\n", h, s);

	while(ReadFile((HANDLE) h, buf, 2048, &r, NULL)) {
		if(!r) break;

		at = 0;
		while(at < r) {
			w = send(s, buf+at, r-at, 0);
			if(w < 0) {
				fprintf(stderr, "os_wrap_read_run write error\n");
				goto end;
			}
			at += w;
		}
	}

	err = GetLastError();
	if(err != ERROR_HANDLE_EOF && err != ERROR_SUCCESS && err != ERROR_BROKEN_PIPE) {
		fprintf(stderr, "os_wrap_read read error: %d\n", err);
	}

end:

	//fprintf(stderr, "ending os_wrap_read %d %d\n", h, s);

	if(!CloseHandle((HANDLE) h)) {
		fprintf(stderr, "os_wrap_read CloseHandle fails: %d\n", (int) GetLastError());
	}

	shutdown(s, SD_BOTH);
	EVUTIL_CLOSESOCKET(s);

	return 0;
}

/* wrap a HANDLE in a SOCKET
 * reading from handle, writing into socket */
static int os_wrap_read(int h)
{
	int pipe[2];
	HANDLE t;

	int res = evutil_socketpair(AF_UNIX, SOCK_STREAM, 0, pipe);
	if(res) {
		fprintf(stderr, "os_wrap_read: evutil_socketpair fails: %d\n", res);
		exit(1);
	}

	hspair_t *p = malloc(sizeof *p);
	if(!p) abort();
	p->h = h;
	p->s = pipe[1];

	t = CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE) os_wrap_read_run, p, 0, NULL);
	if(t == NULL) {
		fprintf(stderr, "os_wrap_read: CreateThread failed\n");
		exit(1);
	}

	//fprintf(stderr, "os_wrap_read: wrapping h %d in socket %d (other socket %d)\n", h, pipe[0], pipe[1]);

	return pipe[0];
}

/* thread for wrapping a HANDLE in a SOCKET:
 * reading from socket, writing into handle */
static DWORD os_wrap_write_run(LPVOID data)
{
	hspair_t *p = data;
	int h = p->h;
	int s = p->s;
	free(p);

	char buf[2048];
	int r, at, w;

	//fprintf(stderr, "entering os_wrap_write %d %d\n", h, s);

	while((r = recv(s, buf, 2048, 0)) > 0) {
		at = 0;
		while(at < r) {
			if(!WriteFile((HANDLE) h, buf+at, r-at, (DWORD *) &w, NULL)) {
				fprintf(stderr, "os_wrap_write_run write error: %d\n", (int) GetLastError());
				goto end;
			}

			if(!w) {
				fprintf(stderr, "os_wrap_write_run zero write on %d???\n", (int) h);
			}

			at += w;
		}
	}

	if(r < 0) {
		fprintf(stderr, "os_wrap_write recv error: %s\n", strerror(errno));
	}

end:

	//fprintf(stderr, "ending os_wrap_write %d %d\n", h, s);

	if(!CloseHandle((HANDLE) h)) {
		fprintf(stderr, "os_wrap_write CloseHandle fails: %d\n", (int) GetLastError());
	}

	shutdown(s, SD_BOTH);
	EVUTIL_CLOSESOCKET(s);

	return 0;
}

/* wrap a HANDLE in a SOCKET
 * reading from socket, writing into handle */
static int os_wrap_write(int h)
{
	int pipe[2];
	HANDLE t;

	int res = evutil_socketpair(AF_UNIX, SOCK_STREAM, 0, pipe);
	if(res) {
		fprintf(stderr, "os_wrap_write: evutil_socketpair fails: %d\n", res);
		exit(1);
	}

	hspair_t *p = malloc(sizeof *p);
	if(!p) abort();
	p->h = h;
	p->s = pipe[1];

	t = CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE) os_wrap_write_run, p, 0, NULL);
	if(t == NULL) {
		fprintf(stderr, "os_wrap_write: CreateThread failed\n");
		exit(1);
	}

	//fprintf(stderr, "os_wrap_write: wrapping h %d in socket %d (other socket %d)\n", h, pipe[0], pipe[1]);

	return pipe[0];
}

static inline int os_net_read(int s, void *buf, int siz)
{
	return recv(s, buf, siz, 0);
}

static inline int os_net_write(int s, void *buf, int siz)
{
	return send(s, buf, siz, 0);
}

static inline void os_closesocket(int s)
{
	shutdown(s, SD_BOTH);
	closesocket(s);
}
