#ifndef _PARSE_H
#define _PARSE_H

#include <sys/time.h>

/* parser result */
typedef enum {
	PARSE_OK=0,
	PARSE_UNREC=1,  /* command not recognized */
	PARSE_DEF=2,    /* unknown kind of default set */
	PARSE_TERM=3,   /* invalid kind of termination */
	PARSE_ID=4,     /* invalid identifier */
	PARSE_SHORT=5,  /* short definition (ran out of tokens) */
	PARSE_FD=6,     /* invalid FD */
	PARSE_REC=7,    /* invalid record size */
	PARSE_PROCEQ=8, /* expected = in proc statement, got something else */
	PARSE_LONG=9,   /* unexpected superfluous tokens */
	PARSE_BIN=10,   /* couldn't unpack hex binary data */
	PARSE_CALL=11,  /* callback returned non-zero */
	PARSE_TIME=12   /* invalid timer delay spec */
} parse_result_t;

/* possible actions after a process or pipe is terminated */
typedef enum {
	TERM_DEFAULT=0,
	TERM_KILL=1,
	TERM_EVENT=2
} termination_t;

/* parsed definition of process input FD */
typedef struct {
	int fd;
	termination_t term;
	int cascade;
} p_proc_infd_t;

/* parsed definition of process output FD */
typedef struct {
	int fd;
	int record; /* record size, -1 if not given (line based) */
	termination_t term;
	int cascade;
} p_proc_outfd_t;

/* pipe source type */
typedef enum {
	PSRC_PROC,
	PSRC_PLUMB,
	PSRC_EV
} p_pipe_src_type_t;

/* pipe source definition */
typedef struct {
	p_pipe_src_type_t type;
	char *proc;
	int fd;
} p_pipe_src_t;

/* pipe target type */
typedef enum {
	PTGT_PROC,
	PTGT_PLUMB,
	PTGT_CTRL
} p_pipe_tgt_type_t;

/* pipe target definition */
typedef struct {
	p_pipe_tgt_type_t type;
	char *proc;
	int fd;
} p_pipe_tgt_t;

/* pipe filter type */
typedef enum {
	FILT_NONE=0,
	FILT_FILTER=1,
	FILT_CUT=2
} filter_type_t;

/* pipe placement type */
typedef enum {
	PPLACE_FIRST,
	PPLACE_LAST
} p_pipe_place_t;

/* declaration for parser callbacks */

int p_set(void *data, char *key, int argc, char **argv);
int p_default_proc_term(void *data, termination_t term);
int p_default_eof_term(void *data, termination_t term);
int p_default_write_term(void *data, termination_t term);
int p_default_src_cascade(void *data, int cascade);
int p_default_tgt_cascade(void *data, int cascade);
int p_proc(void *data, const char *id, termination_t term, int pause, int infdn, const p_proc_infd_t *infds, int outfdn, const p_proc_outfd_t *outfds, int argc, char **argv);
int p_pipe(void *data, const char *id, p_pipe_src_t src, p_pipe_tgt_t tgt, filter_type_t filter, int filterlen, const char *filterdata, int prefixlen, const char *prefixdata, int endchain, p_pipe_place_t place);
int p_endproc(void *data, char *id);
int p_endpipe(void *data, char *id);
int p_unpause(void *data, const char *id);
int p_timer(void *data, const char *timerid, int repeat, struct timeval delay, const char *output);
int p_endtimer(void *data, const char *timerid);

/* parser calls */

parse_result_t parse(void *data, int argc, char **argv);
const char *parse_error(parse_result_t res);

#endif
