#include <sys/socket.h>
#include <netinet/in.h>
#include <netinet/tcp.h>
#include <arpa/inet.h>
#include <errno.h>
#include <netdb.h>
#include <signal.h>

#include "osunix.h"

typedef int socket_t;

static void os_net_init()
{
	signal(SIGPIPE, SIG_IGN);
}

static int os_net_server(int port, char *iface)
{
	int serversock, opt;
	struct sockaddr_in serv_addr;

	serversock = socket(AF_INET, SOCK_STREAM, 0);
	if(serversock < 0) {
		perror("socket");
		exit(1);
	}

	memset(&serv_addr, 0, sizeof(serv_addr));
	serv_addr.sin_family = AF_INET;
	serv_addr.sin_port = htons(port);
	if(!inet_aton(iface, (struct in_addr *) &serv_addr.sin_addr.s_addr)) {
		fprintf(stderr, "Invalid interface address %s\n", iface);
		exit(1);
	}

	opt = 1;
	if(setsockopt(serversock, SOL_SOCKET, SO_REUSEADDR, &opt, sizeof(opt)) < 0) {
		perror("setsockopt");
		exit(1);
	}

	if(bind(serversock, (struct sockaddr *) &serv_addr, sizeof(serv_addr)) < 0) {
		perror("bind");
		exit(1);
	}

	if(listen(serversock, 64) < 0) {
		perror("listen");
		exit(1);
	}

	return serversock;
}

static int os_net_accept(int serv, int *out, struct sockaddr_in *outaddr)
{
	socklen_t clilen = sizeof(*outaddr);

	*out = accept(serv, (struct sockaddr *) outaddr, &clilen);
	if(*out < 0)
		return 1;

	return 0;
}

static void os_net_nodelay(int s)
{
	int opt=1;
	if(setsockopt(s, IPPROTO_TCP, TCP_NODELAY, &opt, sizeof(opt))) {
		perror("setsockopt TCP_NODELAY");
	}
}

static int os_net_close(int s)
{
	return close(s);
}

static int os_stdin_fd()
{
	return 0;
}

static int os_net_client(char *host, int port)
{
	int s;
	struct hostent *name;
	struct sockaddr_in addr;

	s = socket(AF_INET, SOCK_STREAM, 0);

	if(s < 0) {
		perror("socket");
		exit(1);
	}

	name = gethostbyname(host);
	if(!name) {
		fprintf(stderr, "Host name lookup failure: %s\n", host);
		exit(1);
	}

	memset(&addr, 0, sizeof(struct sockaddr_in));
	addr.sin_family = AF_INET;
	addr.sin_port = htons(port);
	memcpy(&addr.sin_addr, name->h_addr, sizeof(addr.sin_addr));

	if(connect(s, (struct sockaddr *) &addr, sizeof(struct sockaddr_in)) < 0) {
		perror("connect failed\n");
		exit(1);
	}

	return s;
}
