#include <windows.h>
#include <winsock2.h>

#include "oswin32.h"

typedef SOCKET socket_t;

static void os_net_init()
{
	int r;
	WSADATA wsaData;

	r = WSAStartup(MAKEWORD(2,0), &wsaData);
	if(r != 0) {
		fprintf(stderr, "WSAStartup failed\n");
		exit(1);
	}
}

static SOCKET os_net_server(int port, char *iface)
{
	int serversock;
	struct sockaddr_in serv_addr;
	unsigned long inaddr;

	serversock = WSASocket(AF_INET, SOCK_STREAM, 0, NULL, 0, WSA_FLAG_OVERLAPPED);
	if(serversock == INVALID_SOCKET) {
		fprintf(stderr, "WSASocket fails: %d\n", WSAGetLastError());
		exit(1);
	}

	memset(&serv_addr, 0, sizeof(serv_addr));
	serv_addr.sin_family = AF_INET;
	serv_addr.sin_port = htons(port);

	inaddr = inet_addr(iface);
	if(inaddr == INADDR_NONE) {
		fprintf(stderr, "Invalid interface address %s\n", iface);
		exit(1);
	}

	serv_addr.sin_addr.s_addr = inaddr;

	if(bind(serversock, (struct sockaddr *) &serv_addr, sizeof(serv_addr)) < 0) {
		perror("bind");
		exit(1);
	}

	if(listen(serversock, 64) < 0) {
		perror("listen");
		exit(1);
	}

	return serversock;
}

static int os_net_accept(SOCKET serv, SOCKET *out, struct sockaddr_in *outaddr)
{
	int clilen = sizeof(*outaddr);

	*out = accept(serv, (struct sockaddr *) outaddr, &clilen);
	if(*out == INVALID_SOCKET)
		return 1;

	return 0;
}

static void os_net_nodelay(SOCKET s)
{
	BOOL opt=1;
	if(setsockopt(s, IPPROTO_TCP, TCP_NODELAY, (void *) &opt, sizeof(opt))) {
		perror("setsockopt TCP_NODELAY");
	}
}

static int os_net_close(int s)
{
	return closesocket(s);
}

static int os_stdin_fd()
{
	return os_wrap_read((int) GetStdHandle(STD_INPUT_HANDLE));
}

static int os_net_client(char *host, int port)
{
	int s;
	struct hostent *name;
	struct sockaddr_in addr;

	s = socket(AF_INET, SOCK_STREAM, 0);

	if(s < 0) {
		perror("socket");
		exit(1);
	}

	name = gethostbyname(host);
	if(!name) {
		fprintf(stderr, "Host name lookup failure: %s\n", host);
		exit(1);
	}

	memset(&addr, 0, sizeof(struct sockaddr_in));
	addr.sin_family = AF_INET;
	addr.sin_port = htons(port);
	memcpy(&addr.sin_addr, name->h_addr, sizeof(addr.sin_addr));

	if(connect(s, (struct sockaddr *) &addr, sizeof(struct sockaddr_in)) < 0) {
		perror("connect failed\n");
		exit(1);
	}

	return s;
}

#define SHUT_RD SD_RECEIVE
#define SHUT_WR SD_SEND
#define SHUT_RDWR SD_BOTH
