#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "tokens.h"
#include "parse.h"

void parg(int argc, char **argv)
{
	int i;
	for(i=0; i<argc; i++) {
		printf(" %s", argv[i]);
	}
	printf("\n");
}

int p_set(void *data, char *key, int argc, char **argv)
{
	printf("SET %s =", key);
	parg(argc, argv);
	tok_setvar(data, key, argc, argv);
	return 0;
}

int p_default_proc_term(void *data, termination_t term)
{
	printf("DEFAULT PROC TERM %d\n", term);
	return 0;
}

int p_default_eof_term(void *data, termination_t term)
{
	printf("DEFAULT EOF TERM %d\n", term);
	return 0;
}

int p_default_write_term(void *data, termination_t term)
{
	printf("DEFAULT WRITE TERM %d\n", term);
	return 0;
}

int p_default_src_cascade(void *data, int cascade)
{
	printf("DEFAULT SRC CASCADE %d\n", cascade);
	return 0;
}

int p_default_tgt_cascade(void *data, int cascade)
{
	printf("DEFAULT TGT CASCADE %d\n", cascade);
	return 0;
}

int p_proc(void *data, const char *id, termination_t term, int pause, int infdn, const p_proc_infd_t *infds, int outfdn, const p_proc_outfd_t *outfds, int argc, char **argv)
{
	printf("PROC %s TERM %d%s INFDS %d OUTFDS %d EXEC", id, term, pause ? " -pause" : "", infdn, outfdn);
	parg(argc, argv);
	int i;
	for(i=0; i<infdn; i++) {
		printf(" INFD%d %d TERM %d CASCADE %d\n", i+1, infds[i].fd, infds[i].term, infds[i].cascade);
	}
	for(i=0; i<outfdn; i++) {
		printf(" OUTFD%d %d RECORD %d TERM %d CASCADE %d\n", i+1, outfds[i].fd, outfds[i].record, outfds[i].term, outfds[i].cascade);
	}
	return 0;
}

int p_pipe(void *data, const char *id, p_pipe_src_t src, p_pipe_tgt_t tgt, filter_type_t filter, int filterlen, const char *filterdata, int prefixlen, const char *prefixdata, int endchain, p_pipe_place_t place)
{
	printf("PIPE %s", id);

	if(place == PPLACE_FIRST)
		printf(" -front");

	if(src.type == PSRC_PROC) {
		printf(" %s/%d ->", src.proc, src.fd);
	} else if(src.type == PSRC_EV) {
		printf(" :ev ->");
	} else if(src.type == PSRC_PLUMB) {
		printf(" :plumb/%d ->", src.fd);
	}

	if(tgt.type == PTGT_PROC) {
		printf(" %s/%d", tgt.proc, tgt.fd);
	} else if(tgt.type == PTGT_CTRL) {
		printf(" :ctrl");
	} else if(tgt.type == PTGT_PLUMB) {
		printf(" :plumb/%d", tgt.fd);
	}

	if(filter) {
		printf(" FILTER %d", filter);
	}

	if(filterlen > 0) {
		printf(" FILTERDATA '");
		fwrite(filterdata, filterlen, 1, stdout);
		printf("'");
	}

	if(prefixlen > 0) {
		printf(" PREFIXDATA '");
		fwrite(prefixdata, prefixlen, 1, stdout);
		printf("'");
	}

	if(endchain) {
		printf(" ENDCHAIN");
	}

	printf("\n");

	return 0;
}

int p_endproc(void *data, char *id)
{
	printf("ENDPROC %s\n", id);
	return 0;
}

int p_endpipe(void *data, char *id)
{
	printf("ENDPIPE %s\n", id);
	return 0;
}

int p_unpause(void *data, const char *id)
{
	printf("UNPAUSE %s\n", id);
	return 0;
}

int p_timer(void *data, const char *timerid, int repeat, struct timeval delay, const char *output)
{
	printf("TIMER AS %s %s%ds %dus : %s\n", timerid, repeat ? "repeat " : "", delay.tv_sec, delay.tv_usec, output);
	return 0;
}

int p_endtimer(void *data, const char *timerid)
{
	printf("ENDTIMER %s\n", timerid);
	return 0;
}

int main(int argc, char *argv[])
{
	tok_t *t = tok_init();

	int line(int tc, char *tv[])
	{
		parse_result_t res = parse(t, tc, tv);
		if(res) {
			printf("PARSE RESULT: %d %s\n", res, parse_error(res));
			return res;
		}

		return 0;
	}

	char buf[1024];
	while(fgets(buf, 1024, stdin)) {
		int r = tok_line(t, buf, line);
		if(r) {
			printf("TOKEN RESULT %d\n", r);
		}
	}

	tok_free(t);

	return 0;
}
