#ifndef _TOKENS_H
#define _TOKENS_H

typedef struct tok_state tok_t;

/* callback for processing tokenized line
 * should return non-zero for error */
typedef int (* tok_line_call_t) (int argc, char **argv);

enum tok_result {
	TOK_OK=0,
	TOK_PARSE_EMPTYVAR=-1,  /* reference to empty variable name */
	TOK_PARSE_STRINGLIT=-2, /* string literal not closed */
	TOK_PARSE_ESCAPE=-3,    /* escape sequence not recognized */
	TOK_PARSE_HEX=-4        /* hex in escape sequence not recognized */
};

/* create tokenizer */
tok_t * tok_init();

/* destroy tokenizer */
void tok_free(tok_t *t);

/* process a line
 * callback will be called with each tokenized command
 * will return one of the tok_result enum (0 on success),
 * or the return value of the first nonzero callback */
int tok_line(tok_t *t, const char *line, tok_line_call_t cb);

/* set variable
 * all data is copied */
void tok_setvar(tok_t *t, const char *key, int tokc, char **tokv);

/* get variable
 * returns token count */
int tok_getvar(tok_t *t, const char *key, char ***tokv);

/* return tokenizer error string */
const char *tok_error(int res);

/* return a concatenated string of tokens
 * uses a static buffer that's overwritten with consecutive calls */
const char *tok_print(int tc, char *tv[]);

#endif
