#ifndef _UTIL_H
#define _UTIL_H

#include "config.h"

#include <stdarg.h>

static inline unsigned int string_hash(const char *str)
{
    register unsigned int h = 2166136261u;
    register const char *s = str;
    while(*s) h = (h * 16777619) ^ *s++;
    return h;
}

#define KEY_T void *
#define VALUE_T void *
#define PREFIX(X) ptrhash ## X
#include "util_hash_h.h"
#undef PREFIX
#undef KEY_T
#undef VALUE_T

ptrhash_t * strhash_new();

/* unordered list of pointers */

typedef void (* ptrset_renum_call_t) (void *ptr, int at);

typedef struct {
	int n;
	int alloc;
	void **p;

	/* called when the index of a pointer is changed */
	ptrset_renum_call_t renum;
} ptrset_t;

void ptrset_init(ptrset_t *set, ptrset_renum_call_t renum);
void ptrset_free(ptrset_t *set);
void ptrset_add(ptrset_t *set, void *ptr);
void ptrset_del_idx(ptrset_t *set, int at);

/* ordered list of pointers */

typedef struct ptrlist_el ptrlist_el_t;

struct ptrlist_el {
	void *ptr;
	ptrlist_el_t *prev;
	ptrlist_el_t *next;
};

typedef struct {
	ptrlist_el_t *first;
	ptrlist_el_t *last;
} ptrlist_t;

void ptrlist_init(ptrlist_t *l);
void ptrlist_free(ptrlist_t *l);
ptrlist_el_t * ptrlist_add_first(ptrlist_t *l, void *ptr);
ptrlist_el_t * ptrlist_add_last(ptrlist_t *l, void *ptr);
void ptrlist_del_el(ptrlist_t *l, ptrlist_el_t *el);

/* kinda-like-sprintf that allocates big enough buffer
 * writes into static buffer (bufprintfbuf),
 * that's overwritten with consecutive calls
 * returns length of message (excluding \0) */
int bufprintf(const char *fmt, ...);
int vbufprintf(const char *fmt, va_list ap);
extern char *bufprintfbuf;

/* string assembling utility
 * works in a static buffer (separate from bufprintf's buffer) */
extern int strasm_len;
extern char *strasm_buf;
void strasm_reset();
void strasm_add(const char *str);

#endif
