/* open addressing hashtable: KEY_T -> VALUE_T */
/* no cached hash, empty key == 0, deleted key == 1 */
/* no memory management (keys are not strdupped or memcpyed) */

typedef KEY_T PREFIX(_key_t);
typedef VALUE_T PREFIX(_value_t);
typedef unsigned int PREFIX(_hash_t);

typedef struct {
	PREFIX(_key_t) key;
	PREFIX(_value_t) value;
} PREFIX(_entry_t);

typedef struct {
	unsigned int mask;
	unsigned int fill;
	unsigned int used; /* valid (not deleted) entries */
	PREFIX(_entry_t) *table;
	
	PREFIX(_hash_t) (*keyhash)(PREFIX(_key_t));
	int (*keyeq)(PREFIX(_key_t), PREFIX(_key_t));
#ifdef HTX_STAT
	unsigned int insert;
	unsigned int del;
	unsigned int resize;
	unsigned int jump;
	unsigned int samehash;
	unsigned int lookup;
#endif
} PREFIX(_t);

PREFIX(_t) *PREFIX(_alloc)(PREFIX(_hash_t) (*keyhash)(PREFIX(_key_t)), int (*keyeq)(PREFIX(_key_t), PREFIX(_key_t)));
PREFIX(_t) *PREFIX(_init)(PREFIX(_t) *ht, PREFIX(_hash_t) (*keyhash)(PREFIX(_key_t)), int (*keyeq)(PREFIX(_key_t), PREFIX(_key_t)));
void PREFIX(_free)(PREFIX(_t) *ht);
PREFIX(_t) *PREFIX(_uninit)(PREFIX(_t) *ht);
PREFIX(_t) *PREFIX(_clear)(PREFIX(_t) *ht);
PREFIX(_t) *PREFIX(_copy)(const PREFIX(_t) *ht);
PREFIX(_t) *PREFIX(_resize)(PREFIX(_t) *ht, unsigned int hint);

/* ht[key] is used */
int PREFIX(_contains)(PREFIX(_t) *ht, PREFIX(_key_t) key);
/* value of ht[key], abort if key is not used */
PREFIX(_value_t) PREFIX(_get)(PREFIX(_t) *ht, PREFIX(_key_t) key);
/* entry of ht[key], NULL if key is not used */
PREFIX(_entry_t) *PREFIX(_getentry)(PREFIX(_t) *ht, PREFIX(_key_t) key);
/* ht[key] = value if key is not used else return ht[key] */
PREFIX(_entry_t) *PREFIX(_insert)(PREFIX(_t) *ht, PREFIX(_key_t) key, PREFIX(_value_t) value);
/* ht[key] = value */
void PREFIX(_set)(PREFIX(_t) *ht, PREFIX(_key_t) key, PREFIX(_value_t) value);
/* delete ht[key] and return its entry, NULL if key is not used */
PREFIX(_entry_t) *PREFIX(_del)(PREFIX(_t) *ht, PREFIX(_key_t) key);
/* delete ht[key] and return its value, abort if key is not used */
PREFIX(_value_t) PREFIX(_pop)(PREFIX(_t) *ht, PREFIX(_key_t) key);
/* delete entry (useful for destructive iteration) */
void PREFIX(_delentry)(PREFIX(_t) *ht, PREFIX(_entry_t) *entry);

/* helper functions */
static inline unsigned int PREFIX(_length)(const PREFIX(_t) *ht) {return ht->used;}
static inline unsigned int PREFIX(_fill)(const PREFIX(_t) *ht) {return ht->fill;}
static inline unsigned int PREFIX(_size)(const PREFIX(_t) *ht) {return ht->mask + 1;}

/* for any entry exactly one returns true */
static inline int PREFIX(_isused)(const PREFIX(_entry_t) *entry) {return entry->key != 0 && entry->key != (PREFIX(_key_t))1;}
static inline int PREFIX(_isempty)(const PREFIX(_entry_t) *entry) {return entry->key == 0;}
static inline int PREFIX(_isdeleted)(const PREFIX(_entry_t) *entry) {return entry->key == (PREFIX(_key_t))1;}


/* first used (useful for iteration) */
static inline PREFIX(_entry_t) *PREFIX(_first)(const PREFIX(_t) *ht)
{
	PREFIX(_entry_t) *entry = 0;
	
	if (ht->used)
		for (entry = ht->table; !PREFIX(_isused)(entry); ++entry);
	return entry;
}

/* next used (useful for iteration) */
static inline PREFIX(_entry_t) *PREFIX(_next)(const PREFIX(_t) *ht, PREFIX(_entry_t) *entry)
{
	while (++entry != ht->table + ht->mask + 1)
		if (PREFIX(_isused)(entry))
			return entry;
	return 0;
}

#ifdef HTX_STAT
void PREFIX(_printstat)(const PREFIX(_t) *ht);
#endif
